@@ -1679,9 +1679,15 @@ gtk_status_icon_padding_changed (GtkStatusIcon *status_icon)
   padding = _gtk_tray_icon_get_padding (GTK_TRAY_ICON (priv->tray_icon));
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    gtk_misc_set_padding (GTK_MISC (priv->image), padding, 0);
+    {
+      gtk_widget_set_margin_left (priv->image, padding);
+      gtk_widget_set_margin_right (priv->image, padding);
+    }
   else
-    gtk_misc_set_padding (GTK_MISC (priv->image), 0, padding);
+    {
+      gtk_widget_set_margin_bottom (priv->image, padding);
+      gtk_widget_set_margin_top (priv->image, padding);
+    }
 }
 
 static void
@@ -1717,11 +1723,13 @@ static void
 gtk_status_icon_fg_changed (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv = status_icon->priv;
-  GdkColor *color;
+  GdkRGBA *rgba;
 
-  g_object_get (priv->tray_icon, "fg-color", &color, NULL);
-  gtk_widget_modify_fg (priv->image, GTK_STATE_NORMAL, color);
-  gdk_color_free (color);
+  g_object_get (priv->tray_icon, "fg-color", &rgba, NULL);
+
+  gtk_widget_override_color (priv->image, GTK_STATE_FLAG_NORMAL, rgba);
+
+  gdk_rgba_free (rgba);
 }
 
 static void
@@ -1731,7 +1739,6 @@ gtk_status_icon_color_changed (GtkTrayIcon   *tray,
 {
   GtkStatusIconPrivate *priv = status_icon->priv;
   const gchar *name;
-  GdkColor *color;
 
   switch (pspec->name[0])
     {
@@ -1753,13 +1760,9 @@ gtk_status_icon_color_changed (GtkTrayIcon   *tray,
     {
       GdkRGBA rgba;
 
-      g_object_get (priv->tray_icon, pspec->name, &color, NULL);
+      g_object_get (priv->tray_icon, pspec->name, &rgba, NULL);
 
-      rgba.red = color->red / 65535.;
-      rgba.green = color->green / 65535.;
-      rgba.blue = color->blue / 65535.;
       rgba.alpha = 1;
-      gdk_color_free (color);
 
       gtk_widget_override_symbolic_color (priv->image, name, &rgba);
     }
@@ -2132,7 +2135,7 @@ gtk_status_icon_get_pixbuf (GtkStatusIcon *status_icon)
  *
  * Since: 2.10
  **/
-G_CONST_RETURN gchar *
+const gchar *
 gtk_status_icon_get_stock (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2164,7 +2167,7 @@ gtk_status_icon_get_stock (GtkStatusIcon *status_icon)
  *
  * Since: 2.10
  **/
-G_CONST_RETURN gchar *
+const gchar *
 gtk_status_icon_get_icon_name (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2917,7 +2920,7 @@ gtk_status_icon_set_title (GtkStatusIcon *status_icon,
  *
  * Since: 2.18
  */
-G_CONST_RETURN gchar *
+const gchar *
 gtk_status_icon_get_title (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2961,6 +2964,17 @@ gtk_status_icon_set_name (GtkStatusIcon *status_icon,
   priv = status_icon->priv;
 
 #ifdef GDK_WINDOWING_X11
-  gtk_window_set_wmclass (GTK_WINDOW (priv->tray_icon), name, name);
+  if (gtk_widget_get_realized (priv->tray_icon))
+    {
+      /* gtk_window_set_wmclass() only operates on non-realized windows,
+       * so temporarily unrealize the tray here
+       */
+      gtk_widget_hide (priv->tray_icon);
+      gtk_widget_unrealize (priv->tray_icon);
+      gtk_window_set_wmclass (GTK_WINDOW (priv->tray_icon), name, name);
+      gtk_widget_show (priv->tray_icon);
+    }
+  else
+    gtk_window_set_wmclass (GTK_WINDOW (priv->tray_icon), name, name);
 #endif
 }
