@@ -22,15 +22,17 @@
 
 #include "config.h"
 
+#include "gtktoolitem.h"
+
 #include <string.h>
 
-#include "gtktoolitem.h"
 #include "gtkmarshalers.h"
 #include "gtktoolshell.h"
 #include "gtkseparatormenuitem.h"
+#include "gtksizerequest.h"
 #include "gtkactivatable.h"
 #include "gtkintl.h"
-#include "gtkmain.h"
+#include "gtkmainprivate.h"
 #include "gtkprivate.h"
 
 
@@ -91,29 +93,27 @@ enum {
   PROP_ACTIVATABLE_USE_ACTION_APPEARANCE
 };
 
-#define GTK_TOOL_ITEM_GET_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTK_TYPE_TOOL_ITEM, GtkToolItemPrivate))
 
 struct _GtkToolItemPrivate
 {
   gchar *tip_text;
   gchar *tip_private;
 
-  guint visible_horizontal : 1;
-  guint visible_vertical : 1;
-  guint homogeneous : 1;
-  guint expand : 1;
-  guint use_drag_window : 1;
-  guint is_important : 1;
+  guint visible_horizontal    : 1;
+  guint visible_vertical      : 1;
+  guint homogeneous           : 1;
+  guint expand                : 1;
+  guint use_drag_window       : 1;
+  guint is_important          : 1;
+  guint use_action_appearance : 1;
 
   GdkWindow *drag_window;
-  
   gchar *menu_item_id;
   GtkWidget *menu_item;
 
   GtkAction *action;
-  gboolean   use_action_appearance;
 };
-  
+
 static void gtk_tool_item_finalize     (GObject         *object);
 static void gtk_tool_item_dispose      (GObject         *object);
 static void gtk_tool_item_parent_set   (GtkWidget       *toolitem,
@@ -132,8 +132,14 @@ static void gtk_tool_item_realize       (GtkWidget      *widget);
 static void gtk_tool_item_unrealize     (GtkWidget      *widget);
 static void gtk_tool_item_map           (GtkWidget      *widget);
 static void gtk_tool_item_unmap         (GtkWidget      *widget);
-static void gtk_tool_item_size_request  (GtkWidget      *widget,
-					 GtkRequisition *requisition);
+static void gtk_tool_item_get_preferred_width
+                                        (GtkWidget      *widget,
+                                         gint           *minimum,
+                                         gint           *natural);
+static void gtk_tool_item_get_preferred_height
+                                        (GtkWidget      *widget,
+                                         gint           *minimum,
+                                         gint           *natural);
 static void gtk_tool_item_size_allocate (GtkWidget      *widget,
 					 GtkAllocation  *allocation);
 
@@ -162,7 +168,7 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
   
   object_class = (GObjectClass *)klass;
   widget_class = (GtkWidgetClass *)klass;
-  
+
   object_class->set_property = gtk_tool_item_set_property;
   object_class->get_property = gtk_tool_item_get_property;
   object_class->finalize     = gtk_tool_item_finalize;
@@ -173,10 +179,13 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
   widget_class->unrealize     = gtk_tool_item_unrealize;
   widget_class->map           = gtk_tool_item_map;
   widget_class->unmap         = gtk_tool_item_unmap;
-  widget_class->size_request  = gtk_tool_item_size_request;
+  widget_class->get_preferred_width = gtk_tool_item_get_preferred_width;
+  widget_class->get_preferred_height = gtk_tool_item_get_preferred_height;
   widget_class->size_allocate = gtk_tool_item_size_allocate;
   widget_class->parent_set    = gtk_tool_item_parent_set;
 
+  gtk_container_class_handle_border_width (GTK_CONTAINER_CLASS (klass));
+
   klass->create_menu_proxy = _gtk_tool_item_create_menu_proxy;
   
   g_object_class_install_property (object_class,
@@ -276,7 +285,9 @@ gtk_tool_item_init (GtkToolItem *toolitem)
 {
   gtk_widget_set_can_focus (GTK_WIDGET (toolitem), FALSE);
 
-  toolitem->priv = GTK_TOOL_ITEM_GET_PRIVATE (toolitem);
+  toolitem->priv = G_TYPE_INSTANCE_GET_PRIVATE (toolitem,
+                                                GTK_TYPE_TOOL_ITEM,
+                                                GtkToolItemPrivate);
 
   toolitem->priv->visible_horizontal = TRUE;
   toolitem->priv->visible_vertical = TRUE;
@@ -317,7 +328,7 @@ static void
 gtk_tool_item_parent_set (GtkWidget   *toolitem,
 			  GtkWidget   *prev_parent)
 {
-  if (GTK_WIDGET (toolitem)->parent != NULL)
+  if (gtk_widget_get_parent (GTK_WIDGET (toolitem)) != NULL)
     gtk_tool_item_toolbar_reconfigured (GTK_TOOL_ITEM (toolitem));
 }
 
@@ -397,20 +408,22 @@ gtk_tool_item_property_notify (GObject    *object,
 static void
 create_drag_window (GtkToolItem *toolitem)
 {
+  GtkAllocation allocation;
   GtkWidget *widget;
   GdkWindowAttr attributes;
-  gint attributes_mask, border_width;
+  gint attributes_mask;
 
   g_return_if_fail (toolitem->priv->use_drag_window == TRUE);
 
   widget = GTK_WIDGET (toolitem);
-  border_width = GTK_CONTAINER (toolitem)->border_width;
+
+  gtk_widget_get_allocation (widget, &allocation);
 
   attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.x = widget->allocation.x + border_width;
-  attributes.y = widget->allocation.y + border_width;
-  attributes.width = widget->allocation.width - border_width * 2;
-  attributes.height = widget->allocation.height - border_width * 2;
+  attributes.x = allocation.x;
+  attributes.y = allocation.y;
+  attributes.width = allocation.width;
+  attributes.height = allocation.height;
   attributes.wclass = GDK_INPUT_ONLY;
   attributes.event_mask = gtk_widget_get_events (widget);
   attributes.event_mask |= (GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
@@ -426,17 +439,17 @@ static void
 gtk_tool_item_realize (GtkWidget *widget)
 {
   GtkToolItem *toolitem;
+  GdkWindow *window;
 
   toolitem = GTK_TOOL_ITEM (widget);
   gtk_widget_set_realized (widget, TRUE);
 
-  widget->window = gtk_widget_get_parent_window (widget);
-  g_object_ref (widget->window);
+  window = gtk_widget_get_parent_window (widget);
+  gtk_widget_set_window (widget, window);
+  g_object_ref (window);
 
   if (toolitem->priv->use_drag_window)
     create_drag_window(toolitem);
-
-  widget->style = gtk_style_attach (widget->style, widget->window);
 }
 
 static void
@@ -485,23 +498,31 @@ gtk_tool_item_unmap (GtkWidget *widget)
 }
 
 static void
-gtk_tool_item_size_request (GtkWidget      *widget,
-			    GtkRequisition *requisition)
+gtk_tool_item_get_preferred_width (GtkWidget *widget,
+                                   gint      *minimum,
+                                   gint      *natural)
 {
-  GtkWidget *child = GTK_BIN (widget)->child;
+  GtkWidget *child;
 
+  *minimum = *natural = 0;
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
   if (child && gtk_widget_get_visible (child))
-    {
-      gtk_widget_size_request (child, requisition);
-    }
-  else
-    {
-      requisition->height = 0;
-      requisition->width = 0;
-    }
-  
-  requisition->width += (GTK_CONTAINER (widget)->border_width) * 2;
-  requisition->height += (GTK_CONTAINER (widget)->border_width) * 2;
+    gtk_widget_get_preferred_width (child, minimum, natural);
+}
+
+static void
+gtk_tool_item_get_preferred_height (GtkWidget *widget,
+                                    gint      *minimum,
+                                    gint      *natural)
+{
+  GtkWidget *child;
+
+  *minimum = *natural = 0;
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
+  if (child && gtk_widget_get_visible (child))
+    gtk_widget_get_preferred_height (child, minimum, natural);
 }
 
 static void
@@ -510,25 +531,24 @@ gtk_tool_item_size_allocate (GtkWidget     *widget,
 {
   GtkToolItem *toolitem = GTK_TOOL_ITEM (widget);
   GtkAllocation child_allocation;
-  gint border_width;
-  GtkWidget *child = GTK_BIN (widget)->child;
+  GtkWidget *child;
 
-  widget->allocation = *allocation;
-  border_width = GTK_CONTAINER (widget)->border_width;
+  gtk_widget_set_allocation (widget, allocation);
 
   if (toolitem->priv->drag_window)
     gdk_window_move_resize (toolitem->priv->drag_window,
-                            widget->allocation.x + border_width,
-                            widget->allocation.y + border_width,
-                            widget->allocation.width - border_width * 2,
-                            widget->allocation.height - border_width * 2);
-  
+                            allocation->x,
+                            allocation->y,
+                            allocation->width,
+                            allocation->height);
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
   if (child && gtk_widget_get_visible (child))
     {
-      child_allocation.x = allocation->x + border_width;
-      child_allocation.y = allocation->y + border_width;
-      child_allocation.width = allocation->width - 2 * border_width;
-      child_allocation.height = allocation->height - 2 * border_width;
+      child_allocation.x = allocation->x;
+      child_allocation.y = allocation->y;
+      child_allocation.width = allocation->width;
+      child_allocation.height = allocation->height;
       
       gtk_widget_size_allocate (child, &child_allocation);
     }
@@ -689,7 +709,7 @@ gtk_tool_item_get_ellipsize_mode (GtkToolItem *tool_item)
   
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_ORIENTATION_HORIZONTAL);
 
-  parent = GTK_WIDGET (tool_item)->parent;
+  parent = gtk_widget_get_parent (GTK_WIDGET (tool_item));
   if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return PANGO_ELLIPSIZE_NONE;
 
@@ -716,7 +736,7 @@ gtk_tool_item_get_icon_size (GtkToolItem *tool_item)
 
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_ICON_SIZE_LARGE_TOOLBAR);
 
-  parent = GTK_WIDGET (tool_item)->parent;
+  parent = gtk_widget_get_parent (GTK_WIDGET (tool_item));
   if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return GTK_ICON_SIZE_LARGE_TOOLBAR;
 
@@ -743,7 +763,7 @@ gtk_tool_item_get_orientation (GtkToolItem *tool_item)
   
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_ORIENTATION_HORIZONTAL);
 
-  parent = GTK_WIDGET (tool_item)->parent;
+  parent = gtk_widget_get_parent (GTK_WIDGET (tool_item));
   if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return GTK_ORIENTATION_HORIZONTAL;
 
@@ -786,7 +806,7 @@ gtk_tool_item_get_toolbar_style (GtkToolItem *tool_item)
   
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_TOOLBAR_ICONS);
 
-  parent = GTK_WIDGET (tool_item)->parent;
+  parent = gtk_widget_get_parent (GTK_WIDGET (tool_item));
   if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return GTK_TOOLBAR_ICONS;
 
@@ -814,7 +834,7 @@ gtk_tool_item_get_relief_style (GtkToolItem *tool_item)
   
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_RELIEF_NONE);
 
-  parent = GTK_WIDGET (tool_item)->parent;
+  parent = gtk_widget_get_parent (GTK_WIDGET (tool_item));
   if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return GTK_RELIEF_NONE;
 
@@ -841,7 +861,7 @@ gtk_tool_item_get_text_alignment (GtkToolItem *tool_item)
   
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_ORIENTATION_HORIZONTAL);
 
-  parent = GTK_WIDGET (tool_item)->parent;
+  parent = gtk_widget_get_parent (GTK_WIDGET (tool_item));
   if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return 0.5;
 
@@ -868,7 +888,7 @@ gtk_tool_item_get_text_orientation (GtkToolItem *tool_item)
   
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_ORIENTATION_HORIZONTAL);
 
-  parent = GTK_WIDGET (tool_item)->parent;
+  parent = gtk_widget_get_parent (GTK_WIDGET (tool_item));
   if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return GTK_ORIENTATION_HORIZONTAL;
 
@@ -879,10 +899,11 @@ gtk_tool_item_get_text_orientation (GtkToolItem *tool_item)
  * gtk_tool_item_get_text_size_group:
  * @tool_item: a #GtkToolItem
  *
- * Returns the size group used for labels in @tool_item. Custom subclasses of
- * #GtkToolItem should call this function and use the size group for labels.
+ * Returns the size group used for labels in @tool_item.
+ * Custom subclasses of #GtkToolItem should call this function
+ * and use the size group for labels.
  *
- * Return value: a #GtkSizeGroup
+ * Return value: (transfer none): a #GtkSizeGroup
  *
  * Since: 2.20
  */
@@ -893,7 +914,7 @@ gtk_tool_item_get_text_size_group (GtkToolItem *tool_item)
   
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), NULL);
 
-  parent = GTK_WIDGET (tool_item)->parent;
+  parent = gtk_widget_get_parent (GTK_WIDGET (tool_item));
   if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return NULL;
 
@@ -1061,8 +1082,7 @@ gtk_tool_item_set_tooltip_text (GtkToolItem *tool_item,
 
   g_return_if_fail (GTK_IS_TOOL_ITEM (tool_item));
 
-  child = GTK_BIN (tool_item)->child;
-
+  child = gtk_bin_get_child (GTK_BIN (tool_item));
   if (child)
     gtk_widget_set_tooltip_text (child, text);
 }
@@ -1085,8 +1105,7 @@ gtk_tool_item_set_tooltip_markup (GtkToolItem *tool_item,
 
   g_return_if_fail (GTK_IS_TOOL_ITEM (tool_item));
 
-  child = GTK_BIN (tool_item)->child;
-
+  child = gtk_bin_get_child (GTK_BIN (tool_item));
   if (child)
     gtk_widget_set_tooltip_markup (child, markup);
 }
@@ -1276,20 +1295,21 @@ gtk_tool_item_retrieve_proxy_menu_item (GtkToolItem *tool_item)
 
 /**
  * gtk_tool_item_get_proxy_menu_item:
- * @tool_item: a #GtkToolItem 
+ * @tool_item: a #GtkToolItem
  * @menu_item_id: a string used to identify the menu item
- * 
+ *
  * If @menu_item_id matches the string passed to
  * gtk_tool_item_set_proxy_menu_item() return the corresponding #GtkMenuItem.
  *
- * Custom subclasses of #GtkToolItem should use this function to update
- * their menu item when the #GtkToolItem changes. That the
- * @menu_item_id<!-- -->s must match ensures that a #GtkToolItem will not
- * inadvertently change a menu item that they did not create.
- * 
- * Return value: The #GtkMenuItem passed to
- * gtk_tool_item_set_proxy_menu_item(), if the @menu_item_id<!-- -->s match.
- * 
+ * Custom subclasses of #GtkToolItem should use this function to
+ * update their menu item when the #GtkToolItem changes. That the
+ * @menu_item_id<!-- -->s must match ensures that a #GtkToolItem
+ * will not inadvertently change a menu item that they did not create.
+ *
+ * Return value: (transfer none): The #GtkMenuItem passed to
+ *     gtk_tool_item_set_proxy_menu_item(), if the @menu_item_id<!-- -->s
+ *     match.
+ *
  * Since: 2.4
  **/
 GtkWidget *
@@ -1322,14 +1342,16 @@ gtk_tool_item_get_proxy_menu_item (GtkToolItem *tool_item,
 void
 gtk_tool_item_rebuild_menu (GtkToolItem *tool_item)
 {
+  GtkWidget *parent;
   GtkWidget *widget;
-  
+
   g_return_if_fail (GTK_IS_TOOL_ITEM (tool_item));
 
   widget = GTK_WIDGET (tool_item);
 
-  if (GTK_IS_TOOL_SHELL (widget->parent))
-    gtk_tool_shell_rebuild_menu (GTK_TOOL_SHELL (widget->parent));
+  parent = gtk_widget_get_parent (widget);
+  if (GTK_IS_TOOL_SHELL (parent))
+    gtk_tool_shell_rebuild_menu (GTK_TOOL_SHELL (parent));
 }
 
 /**
